package be.iminds.ilabt.jfed.espec.util;

import org.testng.annotations.Test;

import static org.testng.Assert.*;
import static org.hamcrest.MatcherAssert.*;
import static org.hamcrest.Matchers.*;

public class LimitedLiveLogImplTest {
    @Test
    public void testLiveLog() {
        LimitedLiveLog log = new LimitedLiveLogImpl(10);
        assertThat(log.getSizeLimit(), is(equalTo(10)));

        log.appendText("test");
        assertThat(log.getSizeLimit(), is(equalTo(10)));
        assertThat(log.getReceivedSize(), is(equalTo(4)));
        assertThat(log.getStoredSize(), is(equalTo(4)));
        assertThat(log.getHeadText(), is(equalTo("test")));
        assertThat(log.getTailText(), is(equalTo("")));
        assertThat(log.getText(), is(equalTo("test")));

        log.appendText("ABCD");
        assertThat(log.getSizeLimit(), is(equalTo(10)));
        assertThat(log.getReceivedSize(), is(equalTo(8)));
        assertThat(log.getStoredSize(), is(equalTo(8)));
        assertThat(log.getHeadText(), is(equalTo("testABCD")));
        assertThat(log.getTailText(), is(equalTo("")));
        assertThat(log.getText(), is(equalTo("testABCD")));

        log.appendText("1234");
        assertThat(log.getSizeLimit(), is(equalTo(10)));
        assertThat(log.getReceivedSize(), is(equalTo(12)));
        assertThat(log.getStoredSize(), is(equalTo(10)));
        assertThat(log.getHeadText(), is(equalTo("testA")));
        assertThat(log.getTailText(), is(equalTo("D1234")));
        assertThat(log.getText(), is(equalTo("testA\n...\n\nD1234")));

        log.appendText("wxyz");
        assertThat(log.getSizeLimit(), is(equalTo(10)));
        assertThat(log.getReceivedSize(), is(equalTo(16)));
        assertThat(log.getStoredSize(), is(equalTo(10)));
        assertThat(log.getHeadText(), is(equalTo("testA")));
        assertThat(log.getTailText(), is(equalTo("4wxyz")));
        assertThat(log.getText(), is(equalTo("testA\n...\n\n4wxyz")));
    }

    @Test
    public void testLiveLogEdgeCaseA() {
        LimitedLiveLog log = new LimitedLiveLogImpl(10);
        assertThat(log.getSizeLimit(), is(equalTo(10)));

        log.appendText("testABCD1234");
        assertThat(log.getSizeLimit(), is(equalTo(10)));
        assertThat(log.getReceivedSize(), is(equalTo(12)));
        assertThat(log.getStoredSize(), is(equalTo(10)));
        assertThat(log.getHeadText(), is(equalTo("testA")));
        assertThat(log.getTailText(), is(equalTo("D1234")));
        assertThat(log.getText(), is(equalTo("testA\n...\n\nD1234")));
    }

    @Test
    public void testLiveLogEdgeCaseB() {
        LimitedLiveLog log = new LimitedLiveLogImpl(10);
        assertThat(log.getSizeLimit(), is(equalTo(10)));

        log.appendText("test");
        assertThat(log.getSizeLimit(), is(equalTo(10)));
        assertThat(log.getReceivedSize(), is(equalTo(4)));
        assertThat(log.getStoredSize(), is(equalTo(4)));
        assertThat(log.getHeadText(), is(equalTo("test")));
        assertThat(log.getTailText(), is(equalTo("")));
        assertThat(log.getText(), is(equalTo("test")));

        log.appendText("ABCD12");
        assertThat(log.getSizeLimit(), is(equalTo(10)));
        assertThat(log.getReceivedSize(), is(equalTo(10)));
        assertThat(log.getStoredSize(), is(equalTo(10)));
        assertThat(log.getHeadText(), is(equalTo("testABCD12")));
        assertThat(log.getTailText(), is(equalTo("")));
        assertThat(log.getText(), is(equalTo("testABCD12")));

        log.appendText("3");
        assertThat(log.getSizeLimit(), is(equalTo(10)));
        assertThat(log.getReceivedSize(), is(equalTo(11)));
        assertThat(log.getStoredSize(), is(equalTo(10)));
        assertThat(log.getHeadText(), is(equalTo("testA")));
        assertThat(log.getTailText(), is(equalTo("CD123")));
        assertThat(log.getText(), is(equalTo("testA\n...\n\nCD123")));
    }
}
